import json


class Parser:
    functions_map = None
    user_defined_anon = None
    spark_instance = None
    spark_functions = None

    def __init__(self, spark_instance, user_defined_anon, spark_functions):
        self.spark_instance = spark_instance
        self.spark_functions = spark_functions
        self.user_defined_anon = json.loads(user_defined_anon) if type(
            user_defined_anon) is str else user_defined_anon
        self.functions_map = {"drop_column": self.anon_drop_column,
                              "replace": self.anon_replace,
                              "replace_with_regex": self.anon_replace_with_regex,
                              "sha256": self.anon_sha256,
                              "filter_row": self.anon_filter_row}

    def anon_drop_column(self, column_name):
        self.spark_instance = self.spark_instance.drop(column_name)

    def anon_replace(self, column_name, replace_to):
        self.spark_instance = self.spark_instance.withColumn(
            column_name, self.spark_functions.lit(replace_to))

    def anon_replace_with_regex(self, column_name, replace_from_regex, replace_to):
        self.spark_instance = self.spark_instance.withColumn(
            column_name, self.spark_functions.regexp_replace(column_name, replace_from_regex, replace_to))

    def anon_sha256(self, column_name):
        self.spark_instance = self.spark_instance.withColumn(
            column_name, self.spark_functions.sha2(self.spark_instance[column_name].cast('String'), 256))

    def anon_filter_row(self, where):
        self.spark_instance = self.spark_instance.where(where)

    def parse(self):
        # add all anons into spark instance
        for current_check in self.user_defined_anon:
            current_name = current_check['method']
            current_parameters = current_check['parameters']
            self.functions_map[current_name](**current_parameters)
        return self.spark_instance



from .Parser import Parser as Parser


class ParserFromDynamoDB:
    functions_map = None
    user_defined_anon = None
    spark_instance = None
    spark_functions = None
    client_error = None
    dataframe_name = None
    dynamo_table = None
    anonymizer = None

    def __init__(self, spark_instance, dataframe_name, dynamo_table, spark_functions, client_error):
        self.spark_instance = spark_instance
        self.dataframe_name = dataframe_name
        self.dynamo_table = dynamo_table
        self.spark_functions = spark_functions
        self.client_error = client_error
        self.__get_anonymizer()

    def __get_anonymizer(self):
        try:
            response = self.dynamo_table.get_item(Key={"dataframe_name": self.dataframe_name})
        except self.client_error as e:
            print(e.response['Error']['Message'])
        else:
            self.anonymizer = response['Item']['anonymizer']

    def parse(self):
        return Parser(self.spark_instance, self.anonymizer, self.spark_functions).parse()



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='pyspark-anonymizer',
    packages=['pyspark_anonymizer'],
    version='0.5',
    license='apache-2.0',
    description='Python library which makes it possible to dynamically mask/anonymize data using JSON string or python dict rules in a PySpark environment.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='wesleywilian',
    url='https://github.com/wesleywilian/pyspark-anonymizer',
    download_url='https://github.com/wesleywilian/pyspark-anonymizer/archive/v0.5.tar.gz',
    keywords=['data anonymizer', 'anon', 'spark', 'data mask', 'mask', 'data masking', 'masking'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
